retry

package
v0.0.0-...-7c007b9 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 3, 2026 License: Apache-2.0 Imports: 4 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func NewExponentialBackOff

func NewExponentialBackOff(ctx context.Context, maxInterval time.Duration) backoff.BackOff

NewExponentialBackOff creates a new ExponentialBackOff object.

func Print

func Print(o backoff.Operation, b backoff.BackOff, logger zerolog.Logger, action string) error

Print displays if an error message has occurred, the time before the next attempt and a resolution message.

Types

This section is empty.

Jump to